use druid::{widget::Controller, Data, Env, Event, EventCtx, KeyEvent, Widget, Lens};

pub struct Key<T> {
    action: Box<dyn Fn(&mut EventCtx, &mut T, &Env, &KeyEvent)>,
}

impl<T: Data> Key<T> {
    pub fn new<F: Fn(&mut EventCtx, &mut T, &Env, &KeyEvent) + 'static>(action: F) -> Self {
        Self {
            action: Box::new(action),
        }
    }
}

impl<T: Data, W: Widget<T>> Controller<T, W> for Key<T> {
    fn event(
        &mut self,
        child: &mut W,
        ctx: &mut EventCtx,
        event: &druid::Event,
        data: &mut T,
        env: &Env,
    ) {
        child.event(ctx, event, data, env);

        if let Event::KeyUp(e) = event {
            (self.action)(ctx, data, env, e);
        }
    }
}

pub struct ImmutLens;

impl<T: Clone> Lens<T, T> for ImmutLens {
    fn with<V, F: FnOnce(&T) -> V>(&self, data: &T, f: F) -> V {
        f(data)
    }

    fn with_mut<V, F: FnOnce(&mut T) -> V>(&self, data: &mut T, f: F) -> V {
        f(&mut data.clone())
    }
}
